#include <stdio.h>
#include <math.h>
#include "km.h"

/* #define M2 7
#define NSTACK 50 */

void dsort4c(unsigned long n, double ra[], double rb[],
	     double rc[], double rd[])
{
  void dindexx(unsigned long n, double arr[], unsigned long indx[]);
  unsigned long j,*iwksp,n2;
  double *wksp;
  
  n2=n+1;
  iwksp=lvector(1,n);
  wksp=dvector(1,n);
  dindexx(n,ra,iwksp);
  for (j=1;j<=n;j++) wksp[j]=ra[j];
  for(j=1;j<=n;j++) ra[j]=wksp[iwksp[n2-j]];
  for (j=1;j<=n;j++) wksp[j]=rb[j];
  for(j=1;j<=n;j++) rb[j]=wksp[iwksp[n2-j]];
  for (j=1;j<=n;j++) wksp[j]=rc[j];
  for(j=1;j<=n;j++) rc[j]=wksp[iwksp[n2-j]];
  for (j=1;j<=n;j++) wksp[j]=rd[j];
  for(j=1;j<=n;j++) rd[j]=wksp[iwksp[n2-j]];
  free_dvector(wksp,1,n);
  free_lvector(iwksp,1,n);
}

void dindexx(unsigned long n, double arr[], unsigned long indx[])
{
  unsigned long i,indxt,ir=n,itemp,j,k,l=1;
  int jstack=0,*istack;
  double a;
  
  istack=ivector(1,NSTACK);
  for (j=1;j<=n;j++) indx[j]=j;
  for(;;){
    if(ir-l < M2) {
      for(j=l+1;j<=ir;j++) {
	indxt=indx[j];
	a=arr[indxt];
	for(i=j-1;i>=1;i--) {
	  if(arr[indx[i]] <= a) break;
	  indx[i+1]=indx[i];
	}
	indx[i+1]=indxt;
      }
      if (jstack == 0) break;
      ir=istack[jstack--];
      l=istack[jstack--];
    } else {
      k=(l+ir) >> 1;
      SWAP(indx[k],indx[l+1]);
      if (arr[indx[l+1]] > arr[indx[ir]]) {
	SWAP(indx[l+1],indx[ir])
      }
      if (arr[indx[l]] > arr[indx[ir]]) {
	SWAP(indx[l],indx[ir])
      }
      if (arr[indx[l+1]] > arr[indx[l]]) {
	SWAP(indx[l+1],indx[l])
      }
      i=l+1;
      j=ir;
      indxt=indx[l];
      a=arr[indxt];
      for(;;) {
	do i++; while (arr[indx[i]] < a);
	do j--; while (arr[indx[j]] > a);
	if(j < i) break;
	SWAP(indx[i],indx[j])
      }
      indx[l]=indx[j];
      indx[j]=indxt;
      jstack += 2;
      if (jstack > NSTACK) nrerror("NSTACK too small.");
      if(ir-i+1 >= j-l) {
	istack[jstack]=ir;
	istack[jstack-1]=i;
	ir=j-1;
      } else {
	istack[jstack]=j-1;
	istack[jstack-1]=l;
	l=i;
      }
    }
  }
  free_ivector(istack,1,NSTACK);
}

/* #define R 0.61803399
#define C (1.0-R) */

double dgolden2(double ax, double bx, double cx, double tol, double *xmin,
		double (*f)(double *,double *,long, double),
		double datar[], double datai[], long n)
     /* bracketed minimum in ax,bx,cx of function f is located with
        precision tol */
{
  double f1,f2,x0,x1,x2,x3;
  
  x0=ax;
  x3=cx;
  if(fabs(cx-bx) > fabs(bx-ax)) {
    x1=bx;
    x2=bx+C*(cx-bx);
  } else {
    x2=bx;
    x1=bx-C*(bx-ax);
  }
  f1=(*f)(datar,datai,n,x1);
  f2=(*f)(datar,datai,n,x2);
  while (fabs(x3-x0) > tol*(fabs(x1)+fabs(x2))) {
    if (f2 < f1) {
      SHFT3(x0,x1,x2,R*x1+C*x3)
	SHFT2(f1,f2,(*f)(datar,datai,n,x2))
    } else {
      SHFT3(x3,x2,x1,R*x2+C*x0)
	SHFT2(f2,f1,(*f)(datar,datai,n,x1))
    }
  }
  if (f1 < f2) {
    *xmin=x1;
    return f1;
  } else {
    *xmin=x2;
    return f2;
  }
}

void four1(float data[], unsigned long nn, int isign)
     /* data[1..2*nn] replaces by DFS, nn must by a power of 2 */
{
  unsigned long n,mmax,m,j,istep,i;
  double wtemp,wr,wpr,wpi,wi,theta;  /* double for recurences */
  float tempr,tempi;
  
  n=nn << 1;
  j=1;
  for (i=1;i<n;i+=2) {              /* bit-reversal section */
    if (j>i) {
      SWAP2(data[j],data[i]);
      SWAP2(data[j+1],data[i+1]);
    }
    m=n >> 1;
    while (m >= 2 && j>m) {
      j -= m;
      m >>= 1;
    }
    j += m;
  }
  /* Danielson-Lanczos section */
  mmax=2;
  while (n > mmax) {                    /* outer ln nn loop */
    istep=mmax << 1;
    theta=isign*(6.28318530717959/mmax);   /* initialize */
    wtemp=sin(0.5*theta);
    wpr = -2.0*wtemp*wtemp;
    wpi=sin(theta);
    wr=1.0;
    wi=0.0;
    for (m=1;m<mmax;m+=2) {          /* two inner loops */
      for (i=m;i<=n;i+=istep) {
	j=i+mmax;                   /* D-L formula */
	tempr=wr*data[j]-wi*data[j+1];
	tempi=wr*data[j+1]+wi*data[j];
	data[j]=data[i]-tempr;
	data[j+1]=data[i+1]-tempi;
	data[i] += tempr;
	data[i+1] += tempi;
      }
      wr=(wtemp=wr)*wpr-wi*wpi+wr;   /* trigonometric recurrence */
      wi=wi*wpr+wtemp*wpi+wi;
    }
    mmax=istep;
  }
}

void dfour4(double datar[], double datai[], long n, double f, double *hr,
	    double *hi)
{
  long i,j,nn;
  double c,s,om,fr,fi;
  double *datar2,*datai2;
  
  datar2=dvector(1,n);
  datai2=dvector(1,n);
  nn=n;
  om=PI2*f;
  datar2[1]=datar[1]/2;
  datai2[1]=datai[1]/2;
  datar2[n]=datar[n]/2;
  datai2[n]=datai[n]/2;
  for(i=2;i<n;i++){
    datar2[i]=datar[i];
    datai2[i]=datai[i];
  }
  while(nn!=1) {
    nn=nn/2;
    c=cos(om*nn);
    s=sin(om*nn);
    for(i=1;i<=nn;i++){
      j=i+nn;
      datar2[i]+=c*datar2[j]-s*datai2[j];
      datai2[i]+=c*datai2[j]+s*datar2[j];
    }
  }
  *hr=datar2[1];
  *hi=datai2[1];
  free_dvector(datar2,1,n);
  free_dvector(datai2,1,n);
}

void laskar(double om, double *ampl, double *ph, double datar[],
	    double datai[], long n)
{
  long i,j,nn;
  double c,s,fr,fi;
  double *datar2,*datai2;
  double iint, rint;
  
  datar2=dvector(1,n);
  datai2=dvector(1,n);
  nn=n;
  datar2[1]=datar[1]/2;
  datai2[1]=datai[1]/2;
  datar2[n]=datar[n]/2;
  datai2[n]=datai[n]/2;
  for(i=2;i<n;i++){
    datar2[i]=datar[i];
    datai2[i]=datai[i];
  }
  while(nn!=1) {
    nn=nn/2;
    c=cos(-om*nn);
    s=sin(-om*nn);
    for(i=1;i<=nn;i++){
      j=i+nn;
      datar2[i]+=c*datar2[j]-s*datai2[j];
      datai2[i]+=c*datai2[j]+s*datar2[j];
    }
  }
  rint=datar2[1]/(n-1);
  iint=datai2[1]/(n-1);
  *ampl=2*sqrt(rint*rint+iint*iint);
  *ph=atan2(iint,rint);
  free_dvector(datar2,1,n);
  free_dvector(datai2,1,n);
}

double dpsden4(double datar[], double datai[], long n, double f)
{
  void dfour4(double datar[], double datai[], long n, double f,
	      double *hr, double *hi);
  double psd,hr=0,hi=0;
  
  dfour4(datar,datai,n,f,&hr,&hi);
  psd=hr*hr+hi*hi;
  return -psd;
}

void spctrm2(double *hh1, double *kk1, float *p, long n)
{
  long j,j2,nn;
  float *w;
  
  nn=n+n;
  w=vector(1,nn);
  for(j=1;j<=n;j++){
    j2=j+j;
    w[j2-1]=hh1[j];
    w[j2]=kk1[j];
  }
  four1(w,n,1);
  for(j=1;j<=n;j++){
    j2=j+j;
    p[j]=SQR(w[j2-1])+SQR(w[j2]);
  }
  free_vector(w,1,nn);
}

void window(long n, double *hh1, double *kk1)
{
  long j;
  double w1,w;
  
  for(j=1;j<=n;j++) {
    w1=(PI2*(j-1))/(n-1);
    w=(1-cos(w1))/2;
    hh1[j]=hh1[j]*w;
    kk1[j]=kk1[j]*w;
  }
}

void window2(long n, double *hh1, double *kk1, double *hh2, double *kk2)
{
  long j;
  double w1,w;
  
  for(j=1;j<=n;j++) {
    w1=(PI2*(j-1))/(n-1);
    w=(1-cos(w1))/2;
    hh2[j]=hh1[j]*w;
    kk2[j]=kk1[j]*w;
  }
}

void bracket(float *spc, long n, double *amp, double *f1, double *f2,
	     double *f3, long *nmax)
{
  long j,l,n2;
  
  n2=n/2;
  l=0;
  if(spc[n]<spc[1] && spc[1]>spc[2]){
    l++;
    amp[l]=spc[1];
    f1[l]=-1.0/n;
    f2[l]=0;
    f3[l]=1.0/n;
  }
  for(j=2;j<=n2;j++)
    if(spc[j-1]<spc[j] && spc[j]>spc[j+1]){
      l++;
      amp[l]=spc[j];
      f1[l]=(j-2)/(1.0*n);
      f2[l]=(j-1)/(1.0*n);
      f3[l]=j/(1.0*n);
    }
  for(j=n2+2;j<=n-1;j++)
    if(spc[j-1]<spc[j] && spc[j]>spc[j+1]){
      l++;
      amp[l]=spc[j];
      f1[l]=(j-n-2)/(1.0*n);
      f2[l]=(j-n-1)/(1.0*n);
      f3[l]=(j-n)/(1.0*n);
    }
  *nmax=l;
}

void shift(int n, int m, double **x, double **y, double **z,
	   double **ax, double **ay, double **az)
{
  int i,j;
  
  for(j=0;j<m;j++)
    for(i=1;i<=n;i++){
      x[j][i]=x[j+1][i];
      y[j][i]=y[j+1][i];
      z[j][i]=z[j+1][i];
      ax[j][i]=ax[j+1][i];
      ay[j][i]=ay[j+1][i];
      az[j][i]=az[j+1][i];
    }
}

void shift2(int n, int m, double **x, double **y, double **z)
{
  int i,j;
  
  for(j=0;j<m;j++)
    for(i=1;i<=n;i++){
      x[j][i]=x[j+1][i];
      y[j][i]=y[j+1][i];
      z[j][i]=z[j+1][i];
    }
}

void shift3(int n, int m, double **x, double **y)
{
  int i,j;
  
  for(j=0;j<m;j++)
    for(i=1;i<=n;i++){
      x[j][i]=x[j+1][i];
      y[j][i]=y[j+1][i];
    }
}

void scale(int p, double *m, double *a, double *ii,
	   double *o, double *oo, double *nt)
{
  int i;
  double pom;
  
  for(i=1;i<=p;i++)
    if(m[i] != 0)
      m[i]=1/m[i];
  pom=PI2/360.0;
  for(i=1;i<=p;i++) {
    a[i]=a[i]*AU;
    ii[i]=ii[i]*pom;
    o[i]=o[i]*pom;
    oo[i]=oo[i]*pom;
    nt[i]=nt[i]*pom;
  }
}

void scale2(int p, double *t, double *a, double *ii,
	    double *o, double *oo, double *nt)
{
  int i;
  double pom;
  
  *t=*t/(365.25*86400.0);
  pom=360.0/PI2;
  for(i=1;i<=p;i++) {
    a[i]=a[i]/AU;
    ii[i]=ii[i]*pom;
    o[i]=o[i]*pom;
    oo[i]=oo[i]*pom;
    nt[i]=nt[i]*pom;
  }
}

void reverse(int n, int m, double **x, double **y, double **z,
	     double **ax, double **ay, double **az)
{
  int i,j;
  double pom;
  
  for(j=0;j<=floor(m/2.);j++)
    for(i=1;i<=n;i++){
      pom=x[j][i];
      x[j][i]=x[m-j][i];
      x[m-j][i]=pom;
      pom=y[j][i];
      y[j][i]=y[m-j][i];
      y[m-j][i]=pom;
      pom=z[j][i];
      z[j][i]=z[m-j][i];
      z[m-j][i]=pom;
      pom=ax[j][i];
      ax[j][i]=ax[m-j][i];
      ax[m-j][i]=pom;
      pom=ay[j][i];
      ay[j][i]=ay[m-j][i];
      ay[m-j][i]=pom;
      pom=az[j][i];
      az[j][i]=az[m-j][i];
      az[m-j][i]=pom;
    }
}

void multimat(double *rot1,double *rot2,double *rot3,double x,
	      double y,double z,double *xx,double *yy,double *zz)
{
  *xx=rot1[1]*x+rot1[2]*y+rot1[3]*z;
  *yy=rot2[1]*x+rot2[2]*y+rot2[3]*z;
  *zz=rot3[1]*x+rot3[2]*y+rot3[3]*z;
}

void gethk(int n, double *e, double *o, double *h, double *k)
{
  int i;
  
  for(i=1;i<=n;i++) {
    h[i]=e[i]*cos(o[i]);
    k[i]=e[i]*sin(o[i]);
  }
}

void gethk2(int ps,double *as,double *os,double *oos,double *nts,double o,
	    double oo, double nt, double *h1, double *k1)
{
  int i;
  double l1,l2,ot,sig;
  
  for(i=1;i<=ps;i++){
    l1=nt+o+oo;
    while(l1>=PI2) l1-=PI2;
    while(l1<0.0) l1+=PI2;
    l2=nts[i]+os[i]+oos[i];
    while(l2>=PI2) l2-=PI2;
    while(l2<0.0) l2+=PI2;
    sig=l2-l1;
    while(sig>=PI2) sig-=PI2;
    while(sig<0.0) sig+=PI2;
    if(sig>=(PI2/2.0)) sig=sig-PI2;
    h1[i]=as[i]/AU*cos(sig);
    k1[i]=as[i]/AU*sin(sig);
  }
}

void gethk3(int n, double *a, double *h, double *k)
{
  int i;
  
  for(i=1;i<=n;i++) {
    h[i]=a[i]/AU;
    k[i]=0;
  }
}

void gethk4(int n, double *ii, double *oo, double *h, double *k)
{
  int i;
  
  for(i=1;i<=n;i++) {
    h[i]=180.*ii[i]/PI*cos(oo[i]);
    k[i]=180.*ii[i]/PI*sin(oo[i]);
  }
}

void gethk5(int n, double *e, double *o, double *oo, double *h, double *k)
{
  int i;
  
  for(i=1;i<=n;i++) {
    h[i]=e[i]*cos(o[i]+oo[i]);
    k[i]=e[i]*sin(o[i]+oo[i]);
  }
}

void filter(int n, int m, double *d, double **h1, double **k1,
	    double *h1f, double *k1f)
{
  int i,j;
  double sh,sk;
  
  for(i=1;i<=n;i++){
    sh=sk=0;
    for(j=-m;j<=m;j++){
      sh+=d[j]*h1[m-j][i];
      sk+=d[j]*k1[m-j][i];
    }
    h1f[i]=sh;
    k1f[i]=sk;
  }
}





