#include<stdio.h>
#include<math.h>
#include "km.h"

#define SKIP 1

#define BIGNUM 1e100 

int main(void){
  char propin[ML], propout[ML], flagf[ML];
  int i,j,k,ntp,bf_flag,nprop;
  int *id,*flag;
  long l,ndata,nstep;
  double *time, **xdata, **ydata; 
  double datasep,dist,tbuf,xbuf,ybuf;  
  double action,minact,maxact,aveact;
  double angle,minang,maxang,aveang;
  FILE *fp;

  /* READ THE PARAMETERS OF FMFT FROM CONFIG */

  if(ctii("binary_files",&bf_flag)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctii("prop_bodies",&ntp)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctil("prop_length",&ndata)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctil("prop_step",&nstep)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctii("prop_number",&nprop)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctid("points_separation",&datasep)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }

  if(cti("prop_in",propin)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(cti("prop_out",propout)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(cti("dist_start",flagf)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }

  /* ALLOCATION OF VARIABLES */
  xdata = dmatrix(1,ntp,1,ndata);
  ydata = dmatrix(1,ntp,1,ndata);
  time = dvector(1,ndata);
  id = ivector(1,ntp);
  flag = ivector(1,ntp);

  /* READS THE FLAG FILE */
  fp=fopen(flagf,"r");
  for(i=1;i<=ntp;i++)
    fscanf(fp,"%d %lg %d",&id[i],&dist,&flag[i]);
  fclose(fp); 

  /* THE MAIN LOOP */
  for(j=1;j<=nprop;j++){

    fp = fopen(propin,"r");

    /* STEP OVER   '(j-1)*nstep*SKIP'   POINTS */
    if(bf_flag){   
      for(l=1;l<=(j-1)*nstep*SKIP;l++){ 
        fread(&tbuf,8,1,fp);
        for(i=1;i<=ntp;i++){
          fread(&xbuf,8,1,fp);
          fread(&ybuf,8,1,fp);
        }
      }
    } else {
      for(l=1;l<=(j-1)*nstep*SKIP;l++){
        fscanf(fp,"%lg",&tbuf);
        for(i=1;i<=ntp;i++) 
          fscanf(fp,"%lg %lg",&xbuf,&ybuf);
      }
    }

    /* READ ndata POINTS */
    if(bf_flag){
      fread(&time[1],8,1,fp);
      for(i=1;i<=ntp;i++){
        fread(&xdata[i][1],8,1,fp);
        fread(&ydata[i][1],8,1,fp);
      }
      for(l=2;l<=ndata;l++){
        for(k=1;k<=SKIP;k++){
          fread(&time[l],8,1,fp);
          for(i=1;i<=ntp;i++){
            fread(&xdata[i][l],8,1,fp);
            fread(&ydata[i][l],8,1,fp);
          }
        }
      }

    } else {
      fscanf(fp,"%lg",&time[1]);
      for(i=1;i<=ntp;i++) 
	fscanf(fp,"%lg %lg",&xdata[i][1],&ydata[i][1]);
      for(l=2;l<=ndata;l++){
        for(k=1;k<=SKIP;k++){
          fscanf(fp,"%lg",&time[l]);
          for(i=1;i<=ntp;i++) 
            fscanf(fp,"%lg %lg",&xdata[i][l],&ydata[i][l]);
        }
      }
    }
  
    printf("%.15lg\n",time[1]/86400./365.25);   

    /* COMPUTE THE MINIMUM, MAXIMUM AND AVERAGE OF ACTION AND ANGLE */
    for(i=1;i<=ntp;i++){	

      if(flag[i] == 0){
        
          printf("%d 0 0 0 0 0 0\n",i);
        
      } else {

	/* INITIALIZATION */
	minact = BIGNUM; maxact = -BIGNUM; aveact = 0;
	minang = BIGNUM; maxang = -BIGNUM; aveang = 0;
	  

	for(l=1;l<=ndata;l++){	
	  action = sqrt(xdata[i][l]*xdata[i][l]+ydata[i][l]*ydata[i][l]);
	  
	  if(action != 0) angle = atan2(ydata[i][l],xdata[i][l]);
	  else angle = 0;
	  
	  if(angle < 0) angle += 2*PI;  /* ATTENTION WITH THE RESONANT CENTER !!! */  
	  
	  if(action < minact) minact = action;
	  if(action > maxact) maxact = action;
	  if(angle < minang) minang = angle;
	  if(angle > maxang) maxang = angle;
	  aveact += action;
	  aveang += angle;
	}
	aveact = aveact/ndata;
	aveang = aveang/ndata;   

	minang = 180.*minang/PI;
	maxang = 180.*maxang/PI;
	aveang = 180.*aveang/PI;

	/* minact = 180.*minact/PI;
	maxact = 180.*maxact/PI;
	aveact = 180.*aveact/PI; */

	/* minact = minact/AU;
	maxact = maxact/AU;
	aveact = aveact/AU; */ 

	printf("%d %.10lg %.10lg %.10lg %.10lg %.10lg %.10lg\n", 
		 i, minact, maxact, aveact, minang, maxang, aveang);
      }

      close(fp);
    }
    printf("\n");
  }
  return 1;
}
   










