/* frequency decomposition by FMFT */

#include <stdio.h>
#include <math.h>
#include "km.h"

#define FMFT_FLAG 3
#define MINFREQ -3.
#define MAXFREQ 3.
#define SKIP 1 

int main(void)
{
  char fmftin[ML], fmftout[ML], flagf[ML];
  int i,j,k,ntp,bf_flag,nft,nfreq;
  int *id,*flag;
  long l,ndata,nstep;  
  double *time, **xdata, **ydata, **output, **input;
  double datasep,dist,tbuf,xbuf,ybuf;
  double freq,amp,phase,dfreq,damp,dphase;
  FILE *fp,*fpi;

  /* READ THE PARAMETERS OF FMFT FROM CONFIG */

  if(ctii("binary_files",&bf_flag)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctii("fmft_bodies",&ntp)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctil("fmft_length",&ndata)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctil("fmft_step",&nstep)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctii("number_of_fmft",&nft)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctii("number_of_freq",&nfreq)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(ctid("points_separation",&datasep)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }

  if(cti("fmft_in",fmftin)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(cti("fmft_out",fmftout)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }
  if(cti("dist_start",flagf)==KO){
    fprintf(stderr,"error in config\n");
    return(KO);
  }

  /* ALLOCATION OF VARIABLES */
  xdata = dmatrix(1,ntp,1,ndata);
  ydata = dmatrix(1,ntp,1,ndata);
  time = dvector(1,ndata);
  input = dmatrix(1,2,1,ndata);
  output = dmatrix(1,3*FMFT_FLAG,1,nfreq);
  id = ivector(1,ntp);
  flag = ivector(1,ntp);

  /* READ THE FLAG FILE */
  fp = fopen(flagf,"r");
  for(i=1;i<=ntp;i++)
    fscanf(fp,"%d %lg %d",&id[i],&dist,&flag[i]);
  fclose(fp);

  /*   for(i=1;i<=ntp;i++)
       flag[i] = 1; */

  /* THE MAIN LOOP */
  for(j=1;j<=nft;j++){
    
    if(bf_flag)  fp=fopen(fmftin,"rb");
    else fp=fopen(fmftin,"r");

    /* STEP OVER   '(j-1)*nstep*SKIP'   POINTS */
    if(bf_flag){   
      for(l=1;l<=(j-1)*nstep*SKIP;l++){ 
	fread(&tbuf,8,1,fp);
	for(i=1;i<=ntp;i++){
	  fread(&xbuf,8,1,fp);
	  fread(&ybuf,8,1,fp);
	}
      }
    } else {
      for(l=1;l<=(j-1)*nstep*SKIP;l++){
	fscanf(fp,"%lg",&tbuf);
	for(i=1;i<=ntp;i++) 
	  fscanf(fp,"%lg %lg",&xbuf,&ybuf);
      }
    }

    /* READ ndata POINTS */
    if(bf_flag){
      fread(&time[1],8,1,fp);
      for(i=1;i<=ntp;i++){
	fread(&xdata[i][1],8,1,fp);
	fread(&ydata[i][1],8,1,fp);
      }
      for(l=2;l<=ndata;l++){
	for(k=1;k<=SKIP;k++){
	  fread(&time[l],8,1,fp);
	  for(i=1;i<=ntp;i++){
	    fread(&xdata[i][l],8,1,fp);
	    fread(&ydata[i][l],8,1,fp);
	  }
	}
      }

    } else {
      fscanf(fp,"%lg",&time[1]);
      for(i=1;i<=ntp;i++) 
	fscanf(fp,"%lg %lg",&xdata[i][1],&ydata[i][1]);
      for(l=2;l<=ndata;l++){
	for(k=1;k<=SKIP;k++){
	  fscanf(fp,"%lg",&time[l]);
	  for(i=1;i<=ntp;i++) 
	    fscanf(fp,"%lg %lg",&xdata[i][l],&ydata[i][l]);
	}
      }
    }

    
    /* PERFORM FMFT FOR ALL ntp TEST PARTICLES */
    for(i=1;i<=ntp;i++){
      
      printf("%d %.15lg\n",id[i],time[1]/86400./365.25);
      
      if(flag[i] == 0){
	
	for(k=1;k<=nfreq;k++)
	  printf("%d 0 0 0 0 0 0\n",k);
	
      } else {
	
	for(l=1;l<=ndata;l++){
	  input[1][l] = xdata[i][l]; 
	  input[2][l] = ydata[i][l];
	}
	
	fmft(output, nfreq, MINFREQ, MAXFREQ, FMFT_FLAG, 
	     input, ndata);
	
	for(k=1;k<=nfreq;k++){
	  freq = output[3*FMFT_FLAG-2][k] *180.*3600./PI / (datasep * SKIP); 
	  /* arsec per unit of datasep */
	  amp = output[3*FMFT_FLAG-1][k];
	  phase = output[3*FMFT_FLAG][k] *180./PI;  /* degrees */
	  if(phase >= 360) phase -= 360.;
	  if(phase < 0) phase += 360.;
	  
	  if(FMFT_FLAG == 1){
	    
	    dfreq = damp = dphase = 0;
	    
	  } else {
	    
	    dfreq = (output[3*FMFT_FLAG-2][k] - output[3*(FMFT_FLAG-1)-2][k])
	      *180.*3600./PI / (datasep * SKIP); 
	    damp = output[3*FMFT_FLAG-1][k] - output[3*(FMFT_FLAG-1)-1][k] ;
	    dphase = (output[3*FMFT_FLAG][k] - output[3*(FMFT_FLAG-1)][k]) *180./PI;  
	    if(dphase >= 180) dphase -= 360.;
	    if(dphase < -180) dphase += 360.;
	  }
	  
	  printf("%d %.15lg %lg %.15lg %lg %.15lg %lg\n", 
		 k, freq, dfreq, amp, damp, phase, dphase);
	}
      }
      printf("\n");
    }
    fclose(fp);
  }
  return(OK);
}
















